<?php

namespace Classifieds\Feature\Http\Controllers;

use Exception;
use Classifieds\Feature\Actions\FeatureWorkflow;
use Classifieds\Feature\Http\Requests\FeatureWorkflowRequest;
use Classifieds\Feature\Http\Resources\FeatureResource;
use Classifieds\Feature\Models\Feature;
use Litepie\Http\Controllers\ActionController as BaseController;

/**
 * Workflow controller class.
 *
 */
class FeatureWorkflowController extends BaseController
{
    /**
     * Action controller function for feature.
     *
     * @param Model $feature
     * @param action next action for the feature.
     *
     * @return Response
     */
    public function __invoke(FeatureWorkflowRequest $request, Feature $feature, $trasition)
    {
        try {
            $request = $request->all();
            $feature = FeatureWorkflow::run($trasition, $feature, $request);
            $data = new FeatureResource($feature);
            return $this->response->message(trans('messages.success.updated', ['Module' => trans('feature::feature.name')]))
                ->code(204)
                ->data(compact('data'))
                ->status('success')
                ->url(guard_url('feature/feature/' . $feature->getRouteKey()))
                ->redirect();
        } catch (Exception $e) {
            return $this->response->message($e->getMessage())
                ->code(400)
                ->status('error')
                ->url(guard_url('feature/feature/' . $feature->getRouteKey()))
                ->redirect();
        }
    }


    /**
     * Get the feature and user for the workflow.
     *
     * @param PublicRequest $request
     * @param string $feature
     * @param string $user
     *
     * @return Response
     */
    public function get(PublicRequest $request, $feature, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }

        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $feature = Feature::findBySignedId($feature);
        $transitions = $this->transitions($feature, $user);
        $user = user();
        Auth::logout();
        return $this->response->setMetaTitle(trans('feature::feature.name') . ' ' . trans('Approvals'))
            ->view('feature::public.feature.workflow')
            ->layout('mobile')
            ->data(compact('feature', 'user', 'transitions'))
            ->output();
    }

    /**
     * Display the workflow form for the feature.
     *
     * @param PublicRequest $request
     * @param string $feature
     * @param string $user
     *
     * @return Response
     */
    public function post(PublicRequest $request, $feature, $user)
    {
        if (!$request->hasValidSignature()) {
            abort(403, 'This url is expired!');
        }
        $transition = $request->transition;
        $auth = Auth::guard()->getProvider()->getModel();
        $user = $auth::findBySignedId($user);
        Auth::guard()->login($user);
        $feature = Feature::findBySignedId($feature);
        $user = user();
        $feature = FeatureWorkflow::run($transition, $feature, $request->all());
        Auth::logout();

        return response()->json(
            [
                'status' => 'success',
                'url' => $request->url(),
            ]
        );
    }

    private function transitions($feature, $user)
    {
        $transitions = [];
        foreach ($feature->workflow()->transitions($feature) as $key => $value) {
            $name = $value->getName();
            $array['url'] = URL::temporarySignedRoute('litecms.feature.workflow', now()->addMinutes(3), [
                'guard' => 'admin',
                'transition' => $name,
                'feature' => $feature->getSignedId(now()->addMinutes(3)),
                'user' => $user->getSignedId(now()->addMinutes(3)),
                'trans' => 'en',
            ]
            );
            $array['name'] = $name;
            $array['key'] = $name;
            $array['form'] = $feature->workflow()->form($value);
            $array['label'] = trans('feature::feature.workflow.' . $name);
            $transitions[] = $array;
        }
        return $transitions;
    }
}
