<?php

namespace Classifieds\Feature\Listeners;

use Illuminate\Support\Facades\Notification;
use Illuminate\Support\Str;
use Classifieds\Feature\Events\FeatureWorkflow as FeatureWorkflowEvent;
use Classifieds\Feature\Notifications\FeatureWorkflow as FeatureWorkflowNotification;
use Litepie\Actions\Concerns\AsAction;

class FeatureWorkflow
{
    use AsAction;

    private $allowedTransitions = [
        'before' => [],
        'after' => ['publish', 'submit', 'approve'],
    ];

    public function handle(FeatureWorkflowEvent $event)
    {
        $function = Str::camel($event->transition);
        return $this->$function($event);
    }

    /**
     * Sends a notification to the client when the $feature is submitted.
     *
     * @param FeatureWorkflowEvent $event The event object.
     * @return void
     */
    public function submit(FeatureWorkflowEvent $event)
    {
        $client = $event->feature->client;
        Notification::send($client, new FeatureWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $feature is published.
     *
     * @param FeatureWorkflowEvent $event The event object.
     * @return void
     */
    public function publish(FeatureWorkflowEvent $event)
    {
        $client = $event->feature->client;
        Notification::send($client, new FeatureWorkflowNotification($event));
    }

    /**
     * Sends a notification to the client when the $feature is approved.
     *
     * @param FeatureWorkflowEvent $event The event object.
     * @return void
     */
    public function approve(FeatureWorkflowEvent $event)
    {
        $client = $event->feature->client;
        Notification::send($client, new FeatureWorkflowNotification($event));
    }

    /**
     * Handles the $feature workflow event as a listener.
     *
     * @param FeatureWorkflowEvent $event The event object.
     * @return void
     */
    public function asListener(FeatureWorkflowEvent $event)
    {
        if (($event->when == 'before' &&
            in_array($event->transition, $this->allowedTransitions['before']) ||
            $event->when == 'after' &&
            in_array($event->transition, $this->allowedTransitions['after']))
        ) {
            return $this->handle($event);
        }
    }
}
